@@ -18,6 +18,8 @@ __docformat__ = "epytext en"
 class automodel(model):
     """Automatically build complete model(s) using template information"""
 
+    _alignment_info = None
+
     # Default variables
     max_ca_ca_distance = 14.0
     max_n_o_distance   = 11.0
@@ -357,6 +359,9 @@ class automodel(model):
             # Do model assessment if requested
             self.assess(atmsel, self.assess_methods, out)
         else:
+            # Do model assessment if requested
+            assess_keys = self.assess(atmsel, self.assess_methods, out)
+
             e = atmsel.energy(output='LONG VIOLATIONS_PROFILE',
                               file=modfile.default(file_id='.V', file_ext='',
                                                    root_name=self.sequence,
@@ -365,9 +370,6 @@ class automodel(model):
             (out['molpdf'], out['pdfterms']) = e
             self.user_after_single_model()
 
-            # Do model assessment if requested
-            assess_keys = self.assess(atmsel, self.assess_methods, out)
-
             # Write the final model; Biso contains the violations profile
             self.write_final_model(filename, assess_keys, out)
 
@@ -376,31 +378,56 @@ class automodel(model):
         self.write(file=filename, model_format=self.output_model_format,
                    extra_data=self.get_extra_model_data(assess_keys, out))
 
+    def _get_residue_from_index(self, ind):
+        """Map an alignment index into a model residue"""
+        r = self.residues[ind - 1]
+        return "%s:%s" % (r.num, r.chain.name)
+
     def get_extra_model_data(self, assess_keys, out):
         """Get extra data to insert into mmCIF or PDB output files"""
+        # Use only templates that aligned with the model
+        knowns = [x for x in self.knowns if self._alignment_info[x] is not None]
         if self.output_model_format == 'MMCIF':
+            def format_template(x):
+                info = self._alignment_info[x]
+                return "%s %s %s %s %s %.3f" % \
+                       (x, info[0][0], info[1][0],
+                        self._get_residue_from_index(info[0][1]),
+                        self._get_residue_from_index(info[1][1]), info[2])
             templates = '\n#\nloop_\n_modeller_template.id\n' \
                         + '_modeller_template.name\n' \
-                        + "\n".join(["%d %s" % (x[0]+1, x[1]) \
-                                    for x in enumerate(self.knowns)])
+                        + '_modeller_template.template_begin\n' \
+                        + '_modeller_template.template_end\n' \
+                        + '_modeller_template.target_begin\n' \
+                        + '_modeller_template.target_end\n' \
+                        + '_modeller_template.pct_seq_id\n' \
+                        + "\n".join(["%d %s" % (x[0]+1, format_template(x[1])) \
+                                    for x in enumerate(knowns)])
             scores = self.get_scores_model_data(assess_keys, out,
                                                 "_modeller.%s %s", "_")
             return '_modeller.sequence ' + self.sequence + scores + templates
         else:
-            templates = "\n" + "\n".join(["REMARK   6 TEMPLATE: %s" % x \
-                                         for x in self.knowns])
+            def format_template(x):
+                info = self._alignment_info[x]
+                return "%s %s - %s MODELS %s - %s AT %.1f%%" \
+                       % (x, info[0][0], info[1][0],
+                          self._get_residue_from_index(info[0][1]),
+                          self._get_residue_from_index(info[1][1]), info[2])
+            templates = "\n" + "\n".join(["REMARK   6 TEMPLATE: %s" \
+                                         % format_template(x) for x in knowns])
             scores = self.get_scores_model_data(assess_keys, out,
                                                 "REMARK   6 %s: %s", " ")
             return 'REMARK   6 SEQUENCE: ' + self.sequence + scores + templates
 
-    def get_scores_model_data(self, assess_keys, out, fmt, ws_replace):
+    def get_scores_model_data(self, assess_keys, out, fmt, ws_replace,
+                              spacer='\n'):
         """Get contribution to extra_data from assessment scores"""
         def get_score(score):
             if isinstance(score, (tuple, list)):
                 score = score[0]
             return "%.5f" % score
         if assess_keys:
-            return "\n" + \
+            return spacer + \
                    "\n".join([fmt % (k.replace(' ', ws_replace),
                                      get_score(out[k])) for k in assess_keys])
         else:
@@ -469,6 +496,29 @@ class automodel(model):
         """Check the alignment for sanity"""
         aln.check()
 
+    def _get_sequence_info(self, aln, tgt, known):
+        start_res = end_res = None
+
+        for pos in aln.positions:
+            target_res = pos.get_residue(tgt)
+            known_res = pos.get_residue(known)
+            if target_res and known_res:
+                end_res = (known_res, target_res)
+                if start_res is None:
+                    start_res = (known_res, target_res)
+        if start_res and end_res:
+            return [("%s:%s" % (res[0].num, res[0].chain.name), res[1].index)
+                    for res in (start_res, end_res)] \
+                   + [known.get_sequence_identity(tgt)]
+
+    def _get_alignment_info(self, aln):
+        """Get and store certain information about the alignment.
+           This is added to the header of any output model."""
+        self._alignment_info = {}
+        for known in self.knowns:
+            self._alignment_info[known] = self._get_sequence_info(aln,
+                                               aln[self.sequence], aln[known])
+
     def homcsr(self, exit_stage):
         """Construct the initial model and restraints"""
         # Check the alignment
@@ -487,6 +537,8 @@ class automodel(model):
 
         self.check_alignment(aln)
 
+        self._get_alignment_info(aln)
+
         # make topology and build/read the atom coordinates:
         self.make_initial_model(aln)
 
